def install_ops():
    import py_hcl.core.expr.add  # noqa: F401
    import py_hcl.core.expr.and_  # noqa: F401
    import py_hcl.core.expr.xor  # noqa: F401
    import py_hcl.core.expr.or_  # noqa: F401
    import py_hcl.core.stmt.connect  # noqa: F401
    import py_hcl.core.expr.field  # noqa: F401
    import py_hcl.core.expr.slice  # noqa: F401
    import py_hcl.core.expr.index  # noqa: F401
    import py_hcl.core.expr.convert  # noqa: F401
    import py_hcl.core.expr.extend  # noqa: F401
